/*
    SDL - Simple DirectMedia Layer
    Copyright (C) 1997, 1998  Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Sam Lantinga
    5635-34 Springhouse Dr.
    Pleasanton, CA 94588 (USA)
    slouken@devolution.com
*/

#ifdef SAVE_RCSID
static char rcsid =
 "@(#) $Id: SDL_keyboard.c,v 1.5 1998/09/13 16:56:38 slouken Exp $";
#endif

/* General keyboard handling code for SDL */

#include <stdio.h>
#include <ctype.h>

#include "SDL_events.h"
#include "SDL_events_c.h"
#include "SDL_sysevents.h"

#define tocntrl(x)	((x)-'a'+1)	/* From [a-z] to [Ctrl-A - Ctrl-Z] */

/* Global keystate information */
static Uint8  SDL_KeyState[256];
SDLMod SDL_ModState;

static char *keynames[256];		/* Array of keycode names */
static SDLKey shifted[128];		/* Array of shifted key mappings */
static SDLKey control[128];		/* Array of control key mappings */


/* Public functions */
int SDL_KeyboardInit(void)
{
	Uint16 i;

	/* Initialize the tables */
	SDL_ModState = (SDLMod)0;
	for ( i=0; i<SDL_TABLESIZE(keynames, char *); ++i )
		keynames[i] = NULL;
	for ( i=0; i<SDL_TABLESIZE(control, SDLKey); ++i )
		control[i] = (SDLKey)i;
	for ( i=0; i<SDL_TABLESIZE(shifted, SDLKey); ++i )
		shifted[i] = (SDLKey)i;
	for ( i=0; i<SDL_TABLESIZE(SDL_KeyState, Uint8); ++i )
		SDL_KeyState[i] = SDL_RELEASED;
	SDL_InitOSKeymap();

	/* Fill in the blanks in keynames */
	keynames[SDLK_BACKSPACE] = "backspace";
	keynames[SDLK_TAB] = "tab";
	keynames[SDLK_CLEAR] = "clear";
	keynames[SDLK_RETURN] = "return";
	keynames[SDLK_ESCAPE] = "escape";
	keynames[SDLK_SPACE] = "space";
	keynames[SDLK_QUOTE] = "'";
	keynames[SDLK_COMMA] = ",";
	keynames[SDLK_MINUS] = "-";
	keynames[SDLK_PERIOD] = ".";
	keynames[SDLK_SLASH] = "/";
	keynames[SDLK_0] = "0";
	keynames[SDLK_1] = "1";
	keynames[SDLK_2] = "2";
	keynames[SDLK_3] = "3";
	keynames[SDLK_4] = "4";
	keynames[SDLK_5] = "5";
	keynames[SDLK_6] = "6";
	keynames[SDLK_7] = "7";
	keynames[SDLK_8] = "8";
	keynames[SDLK_9] = "9";
	keynames[SDLK_SEMICOLON] = ";";
	keynames[SDLK_EQUALS] = "=";
	keynames[SDLK_KP0] = "[0]";
	keynames[SDLK_KP1] = "[1]";
	keynames[SDLK_KP2] = "[2]";
	keynames[SDLK_KP3] = "[3]";
	keynames[SDLK_KP4] = "[4]";
	keynames[SDLK_KP5] = "[5]";
	keynames[SDLK_KP6] = "[6]";
	keynames[SDLK_KP7] = "[7]";
	keynames[SDLK_KP8] = "[8]";
	keynames[SDLK_KP9] = "[9]";
	keynames[SDLK_KP_PERIOD] = "[.]";
	keynames[SDLK_KP_DIVIDE] = "[/]";
	keynames[SDLK_KP_MULTIPLY] = "[*]";
	keynames[SDLK_KP_MINUS] = "[-]";
	keynames[SDLK_KP_PLUS] = "[+]";
	keynames[SDLK_KP_ENTER] = "enter";
	keynames[SDLK_KP_EQUALS] = "equals";
	keynames[SDLK_LEFTBRACKET] = "[";
	keynames[SDLK_BACKSLASH] = "\\";
	keynames[SDLK_RIGHTBRACKET] = "]";
	keynames[SDLK_BACKQUOTE] = "`";
	keynames[SDLK_a] = "a";
	keynames[SDLK_b] = "b";
	keynames[SDLK_c] = "c";
	keynames[SDLK_d] = "d";
	keynames[SDLK_e] = "e";
	keynames[SDLK_f] = "f";
	keynames[SDLK_g] = "g";
	keynames[SDLK_h] = "h";
	keynames[SDLK_i] = "i";
	keynames[SDLK_j] = "j";
	keynames[SDLK_k] = "k";
	keynames[SDLK_l] = "l";
	keynames[SDLK_m] = "m";
	keynames[SDLK_n] = "n";
	keynames[SDLK_o] = "o";
	keynames[SDLK_p] = "p";
	keynames[SDLK_q] = "q";
	keynames[SDLK_r] = "r";
	keynames[SDLK_s] = "s";
	keynames[SDLK_t] = "t";
	keynames[SDLK_u] = "u";
	keynames[SDLK_v] = "v";
	keynames[SDLK_w] = "w";
	keynames[SDLK_x] = "x";
	keynames[SDLK_y] = "y";
	keynames[SDLK_z] = "z";
	keynames[SDLK_DELETE] = "delete";
	keynames[SDLK_F1] = "f1";
	keynames[SDLK_F2] = "f2";
	keynames[SDLK_F3] = "f3";
	keynames[SDLK_F4] = "f4";
	keynames[SDLK_F5] = "f5";
	keynames[SDLK_F6] = "f6";
	keynames[SDLK_F7] = "f7";
	keynames[SDLK_F8] = "f8";
	keynames[SDLK_F9] = "f9";
	keynames[SDLK_F10] = "f10";
	keynames[SDLK_F11] = "f11";
	keynames[SDLK_F12] = "f12";
	keynames[SDLK_F13] = "f13";
	keynames[SDLK_F14] = "f14";
	keynames[SDLK_F15] = "f15";
	keynames[SDLK_PAUSE] = "pause";
	keynames[SDLK_NUMLOCK] = "numlock";
	keynames[SDLK_UP] = "up";
	keynames[SDLK_DOWN] = "down";
	keynames[SDLK_RIGHT] = "right";
	keynames[SDLK_LEFT] = "left";
	keynames[SDLK_DOWN] = "down";
	keynames[SDLK_INSERT] = "insert";
	keynames[SDLK_HOME] = "home";
	keynames[SDLK_END] = "end";
	keynames[SDLK_PAGEUP] = "page up";
	keynames[SDLK_PAGEDOWN] = "page down";
	keynames[SDLK_CAPSLOCK] = "caps lock";
	keynames[SDLK_SCROLLOCK] = "scroll lock";
	keynames[SDLK_RSHIFT] = "right shift";
	keynames[SDLK_LSHIFT] = "left shift";
	keynames[SDLK_RCTRL] = "right ctrl";
	keynames[SDLK_LCTRL] = "left ctrl";
	keynames[SDLK_RALT] = "right alt";
	keynames[SDLK_LALT] = "left alt";
	keynames[SDLK_RMETA] = "right meta";
	keynames[SDLK_LMETA] = "left meta";
	keynames[SDLK_HELP] = "help";
	keynames[SDLK_PRINT] = "print screen";
	keynames[SDLK_SYSREQ] = "sys req";
	keynames[SDLK_MENU] = "menu";
	keynames[SDLK_BREAK] = "break";
	keynames[SDLK_EURO] = "euro";
	keynames[SDLK_POWER] = "power";

	/* Fill in the blanks in control array */
	for ( i=SDLK_a; i<=SDLK_z; ++i )
		control[i] = (SDLKey)tocntrl(i);

	/* Fill in the blanks in shifted array */
	for ( i=SDLK_a; i<=SDLK_z; ++i )
		shifted[i] = (SDLKey)toupper(i);
	shifted[SDLK_BACKQUOTE] = (SDLKey)'~';
	shifted[SDLK_1] = (SDLKey)'!';
	shifted[SDLK_2] = (SDLKey)'@';
	shifted[SDLK_3] = (SDLKey)'#';
	shifted[SDLK_4] = (SDLKey)'$';
	shifted[SDLK_5] = (SDLKey)'%';
	shifted[SDLK_6] = (SDLKey)'^';
	shifted[SDLK_7] = (SDLKey)'&';
	shifted[SDLK_8] = (SDLKey)'*';
	shifted[SDLK_9] = (SDLKey)'(';
	shifted[SDLK_0] = (SDLKey)')';
	shifted[SDLK_MINUS] = (SDLKey)'_';
	shifted[SDLK_EQUALS] = (SDLKey)'+';
	shifted[SDLK_LEFTBRACKET] = (SDLKey)'{';
	shifted[SDLK_RIGHTBRACKET] = (SDLKey)'}';
	shifted[SDLK_BACKSLASH] = (SDLKey)'|';
	shifted[SDLK_SEMICOLON] = (SDLKey)':';
	shifted[SDLK_QUOTE] = (SDLKey)'"';
	shifted[SDLK_COMMA] = (SDLKey)'<';
	shifted[SDLK_PERIOD] = (SDLKey)'>';
	shifted[SDLK_SLASH] = (SDLKey)'?';

	/* Done.  Whew. */
	return(0);
}

Uint8 * SDL_GetKeyState (int *numkeys)
{
	if ( numkeys != (int *)0 )
		*numkeys = 256;
	return(SDL_KeyState);
}
SDLMod SDL_GetModState (void)
{
	return(SDL_ModState);
}

Uint8 SDL_SymToASCII (SDL_keysym *keysym, char **symname)
{
	Uint8 symval, use_shift;

	if ( symname != NULL )
		*symname = keynames[keysym->sym];

	/* We rely on syms below 128 to be in their ASCII representations */
	if ( keysym->sym > 127 )
		return(0);

	/* Perform the appropriate state mapping */
	symval = keysym->sym;
	if ( (symval >= SDLK_KP0) && (symval <= SDLK_KP_EQUALS) ) {
		switch (symval) {
			case SDLK_KP0:
			case SDLK_KP1:
			case SDLK_KP2:
			case SDLK_KP3:
			case SDLK_KP4:
			case SDLK_KP5:
			case SDLK_KP6:
			case SDLK_KP7:
			case SDLK_KP8:
			case SDLK_KP9:
				if ( keysym->mod & KMOD_NUM )
					symval = symval-SDLK_KP0+'0';
				else
					symval = 0;
				break;
			case SDLK_KP_PERIOD:
				if ( keysym->mod & KMOD_NUM )
					symval = '.';
				else
					symval = 127;	/* Delete */
				break;
			case SDLK_KP_DIVIDE:
				symval = '/';
				break;
			case SDLK_KP_MULTIPLY:
				symval = '*';
				break;
			case SDLK_KP_MINUS:
				symval = '-';
				break;
			case SDLK_KP_PLUS:
				symval = '+';
				break;
			case SDLK_KP_ENTER:
				symval = '\r';
				break;
			case SDLK_KP_EQUALS:
				symval = '=';
				break;
		}
	} else
	if ( (keysym->mod & KMOD_CTRL) && isalpha(symval) )
		symval = control[symval];
	else
	if ( (keysym->mod & KMOD_SHIFT) || (keysym->mod & KMOD_CAPS) ) {
		use_shift = ((keysym->mod & KMOD_SHIFT) ? 0x01 : 0x00);
		if ( (symval >= SDLK_a) && (symval <= SDLK_z) ) {
			use_shift ^= ((keysym->mod & KMOD_CAPS) ? 0x01 : 0x00);
		}
		if ( use_shift )
			symval = shifted[symval];
	}
	return(symval);
}

/* These are global for SDL_eventloop.c */
int SDL_PrivateKeyboard(Uint8 state, SDLKey sym)
{
	SDL_Event event;
	int posted;
	Uint8 modstate;
	SDL_keysym keysym;

	/* Don't do anything if there's no sym */
	if ( ! sym )
		return(0);

	/* Set up the keysym */
	modstate = (Uint8)SDL_ModState;
	if ( state == SDL_PRESSED ) {
		keysym.mod = (SDLMod)modstate;
		switch (sym) {
			case SDLK_NUMLOCK:
				modstate ^= KMOD_NUM;
				if ( ! (modstate&KMOD_NUM) )
					state = SDL_RELEASED;
				keysym.mod = (SDLMod)modstate;
				break;
			case SDLK_CAPSLOCK:
				modstate ^= KMOD_CAPS;
				if ( ! (modstate&KMOD_CAPS) )
					state = SDL_RELEASED;
				keysym.mod = (SDLMod)modstate;
				break;
			case SDLK_LCTRL:
				modstate |= KMOD_LCTRL;
				break;
			case SDLK_RCTRL:
				modstate |= KMOD_RCTRL;
				break;
			case SDLK_LSHIFT:
				modstate |= KMOD_LSHIFT;
				break;
			case SDLK_RSHIFT:
				modstate |= KMOD_RSHIFT;
				break;
			case SDLK_LALT:
				modstate |= KMOD_LALT;
				break;
			case SDLK_RALT:
				modstate |= KMOD_RALT;
				break;
			default:
				break;
		}
	} else {
		switch (sym) {
			case SDLK_NUMLOCK:
			case SDLK_CAPSLOCK:
				/* Only send keydown events */
				return(0);
			case SDLK_LCTRL:
				modstate &= ~KMOD_LCTRL;
				break;
			case SDLK_RCTRL:
				modstate &= ~KMOD_RCTRL;
				break;
			case SDLK_LSHIFT:
				modstate &= ~KMOD_LSHIFT;
				break;
			case SDLK_RSHIFT:
				modstate &= ~KMOD_RSHIFT;
				break;
			case SDLK_LALT:
				modstate &= ~KMOD_LALT;
				break;
			case SDLK_RALT:
				modstate &= ~KMOD_RALT;
				break;
			default:
				break;
		}
		keysym.mod = (SDLMod)modstate;
	}
	keysym.sym = sym;

	/* Figure out what type of event this is */
	switch (state) {
		case SDL_PRESSED:
			event.type = SDL_KEYDOWN;
			break;
		case SDL_RELEASED:
			event.type = SDL_KEYUP;
			break;
		default:
			/* Invalid state -- bail */
			return(0);
	}
	posted = 0;

	/* Post the event, if desired */
	if ( SDL_ProcessEvents[event.type] == SDL_ENABLE ) {
		event.key.state = state;
		event.key.keysym = keysym;
		if ( (SDL_EventOK == NULL) || SDL_EventOK(&event) ) {
			posted = 1;
			SDL_PushEvent(&event);
		}
	}
	/* Update internal keyboard state */
	SDL_ModState = (SDLMod)modstate;
	SDL_KeyState[sym] = state;
	return(posted);
}
