/*
    SDL - Simple DirectMedia Layer
    Copyright (C) 1997, 1998  Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Sam Lantinga
    5635-34 Springhouse Dr.
    Pleasanton, CA 94588 (USA)
    slouken@devolution.com
*/

#ifdef SAVE_RCSID
static char rcsid =
 "@(#) $Id: SDL_syswm.c,v 1.5 1998/09/14 00:10:27 slouken Exp $";
#endif

#include <stdlib.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "SDL_version.h"
#include "SDL_error.h"
#include "SDL_video.h"
#include "SDL_syswm.h"
#include "SDL_syswm_c.h"
#include "SDL_lowvideo.h"

Uint8   *SDL_iconcolors = NULL;

void SDL_SYS_SetWMIcon(SDL_Surface *screen, SDL_Surface *icon, Uint8 *mask)
{
	SDL_Surface *sicon;
	XWMHints *wmhints;
	XImage   *icon_image;
	Pixmap    icon_pixmap;
	Pixmap    mask_pixmap;
	GC        GC;
	XGCValues GCvalues;
	int       i, b;
	Uint8    *LSBmask, *color_tried;

	/* Create an image in the screen format */
	sicon = SDL_AllocSurface(SDL_SWSURFACE, icon->w, icon->h,
					screen->format->BitsPerPixel,
						screen->format->Rmask,
						screen->format->Gmask,
						screen->format->Bmask, 0);
	if ( sicon == NULL ) {
		return;
	}
	if ( screen->format->palette ) {
		memcpy(sicon->format->palette->colors,
			screen->format->palette->colors,
			screen->format->palette->ncolors*sizeof(SDL_Color));
	}
	if ( icon->dstfmt != NULL ) {
		/* FIXME: Save the blit mapping! */;
	}
	if ( SDL_MapSurface(icon, sicon->format) < 0 )
		goto done;
	if ( SDL_BlitSurface(icon, NULL, sicon, NULL) < 0 )
		goto done;

	/* Lock down the colors used in the colormap */
	color_tried = NULL;
	if ( sicon->format->BitsPerPixel == 8 ) {
		SDL_Palette *palette;
		Uint8 *p;
		XColor wanted;

		palette = sicon->format->palette;
		color_tried = (Uint8 *)malloc(palette->ncolors);
		SDL_iconcolors = (Uint8 *)malloc(palette->ncolors);
		if ( SDL_iconcolors == NULL ) {
			goto done;
		}
		memset(color_tried, 0, palette->ncolors);
		memset(SDL_iconcolors, 0, palette->ncolors);

		p = (Uint8 *)sicon->pixels; 
		for ( i = sicon->w*sicon->h; i > 0; --i, ++p ) {
			if ( ! color_tried[*p] ) {
				wanted.pixel = *p;
				wanted.red   = (palette->colors[*p].r<<8);
				wanted.green = (palette->colors[*p].g<<8);
				wanted.blue  = (palette->colors[*p].b<<8);
				wanted.flags = (DoRed|DoGreen|DoBlue);
				if (XAllocColor(SDL_Display,
						SDL_DisplayColormap, &wanted)) {
					++SDL_iconcolors[wanted.pixel];
				}
				color_tried[*p] = 1;
			}
		}
	}
	if ( color_tried != NULL ) {
		free(color_tried);
	}

	/* Translate mask data to LSB order and set the icon mask */
	i = (sicon->w/8)*sicon->h;
	LSBmask = (Uint8 *)malloc(i);
	if ( LSBmask == NULL ) {
		goto done;
	}
	memset(LSBmask, 0, i);
	while ( --i >= 0 ) {
		for ( b=0; b<8; ++b )
			LSBmask[i] |= (((mask[i]>>b)&0x01)<<(7-b));
	}
	mask_pixmap = XCreatePixmapFromBitmapData(SDL_Display, SDL_Window,
					LSBmask, sicon->w, sicon->h, 1L, 0L, 1);

	/* Transfer the image to an X11 pixmap */
	icon_image = XCreateImage(SDL_Display, 
			DefaultVisual(SDL_Display, DefaultScreen(SDL_Display)),
			DefaultDepth(SDL_Display, DefaultScreen(SDL_Display)),
			ZPixmap, 0, (char *)sicon->pixels, sicon->w, sicon->h,
			((sicon->format)->BytesPerPixel == 3) ? 32 :
				(sicon->format)->BytesPerPixel*8, 0);
	icon_pixmap = XCreatePixmap(SDL_Display, DefaultRootWindow(SDL_Display),
			sicon->w, sicon->h,
			DefaultDepth(SDL_Display, DefaultScreen(SDL_Display)));
	GC = XCreateGC(SDL_Display, icon_pixmap, 0, &GCvalues);
	XPutImage(SDL_Display, icon_pixmap, GC, icon_image,
					0, 0, 0, 0, sicon->w, sicon->h);
	XFreeGC(SDL_Display, GC);
	XDestroyImage(icon_image);
	free(LSBmask);
	sicon->pixels = NULL;

	/* Set the window icon to the pixmap */
	wmhints = XAllocWMHints();
	wmhints->flags = (IconPixmapHint | IconMaskHint);
	wmhints->icon_pixmap = icon_pixmap;
	wmhints->icon_mask = mask_pixmap;
	XSetWMHints(SDL_Display, SDL_Window, wmhints);
	XFree(wmhints);
	XSync(SDL_Display, False);

  done:
	SDL_FreeSurface(sicon);
	SDL_UnmapSurface(icon);
	return;
}

void SDL_SYS_SetWMCaption(SDL_Surface *screen,
				const char *title, const char *icon)
{
	XTextProperty titleprop, iconprop;

	if ( title != NULL ) {
		XStringListToTextProperty((char **)&title, 1, &titleprop);
		XSetWMName(SDL_Display, SDL_Window, &titleprop);
		XFree(titleprop.value);
	}
	if ( icon != NULL ) {
		XStringListToTextProperty((char **)&icon, 1, &iconprop);
		XSetWMIconName(SDL_Display, SDL_Window, &iconprop);
		XFree(iconprop.value);
	}
	XSync(SDL_Display, False);
}

/* If 'info' is the right version, this function fills it and returns 1.
   Otherwise, in case of a version mismatch, it returns -1.
*/
int SDL_SYS_GetWMInfo(SDL_SysWMinfo *info)
{
	if ( info->version.major <= SDL_MAJOR_VERSION ) {
		info->display = SDL_Display;
		info->window = SDL_Window;
		return(1);
	} else {
		SDL_SetError("Application not compiled with SDL %d.%d\n",
					SDL_MAJOR_VERSION, SDL_MINOR_VERSION);
		return(-1);
	}
}
