/*
    WINMAIN.C, placed in the public domain by Sam Lantinga  4/13/98

    The WinMain function -- calls your program's main() function 
*/

#ifdef SAVE_RCSID
static char rcsid =
 "@(#) $Id: winmain.c,v 1.4 1998/09/11 12:13:40 slouken Exp $";
#endif

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <windows.h>

#include "SDL.h"

/* For the application code */
static char **argv;
static int    argc;

/* From the SDL library code */
SDL_DEFUN(int, SDL_RegisterApp, (char *name, UINT style, HINSTANCE hInst))


/* This is where execution begins */
int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR szCmdLine, int sw)
{
	char *cmdline;
	char *appname;
	int n;
	char *ptr;

	/* Start up DDHELP.EXE before opening any files, so DDHELP doesn't
	   keep them open.  This is a hack.. hopefully it will be fixed 
	   someday.  DDHELP.EXE starts up the first time DDRAW.DLL is loaded.
	 */
	FreeLibrary(LoadLibrary("DDRAW.DLL"));

	/* Redirect standard input and standard output */
	freopen("stdout.txt", "w", stdout);
	setvbuf(stdout, NULL, _IOLBF, BUFSIZ);	/* Line buffered */
	freopen("stderr.txt", "w", stderr);
	setbuf(stderr, NULL);			/* No buffering */

	/* Grab the command line (use malloc since alloca isn't portable) */
	/* We leak it, but that's okay, it's cleaned up  on process exit */
	ptr = GetCommandLine();
	if ( (cmdline=(char *)malloc(strlen(ptr)+1)) == NULL ) {
		/* Yuck, out of memory at startup */
		return(FALSE);
	}
	strcpy(cmdline, ptr);

	/* Parse it into argv and argc */
	argc = 0;
	for ( ptr = cmdline; *ptr; ) {
		/* Skip leading whitespace */
		while ( *ptr && isspace(*ptr) ) ++ptr;
		/* Skip over argument */
		if ( *ptr && (*ptr == '"') ) {
			++ptr;
			if ( *ptr )
				++argc;
			while ( *ptr && (*ptr != '"') ) ++ptr;
		} else {
			if ( *ptr )
				++argc;
			while ( *ptr && ! isspace(*ptr) ) ++ptr;
		}
		if ( *ptr )
			++ptr;
	}
	if ( (argv=(char **)malloc((argc+1)*sizeof(char *))) == NULL ) {
		/* Yuck, out of memory at startup */
		return(FALSE);
	}
	n = 0;
	for ( ptr = cmdline; *ptr; ) {
		/* Skip leading whitespace */
		while ( *ptr && isspace(*ptr) ) ++ptr;
		/* Skip over argument */
		if ( *ptr && (*ptr == '"') ) {
			++ptr;
			if ( *ptr )
				argv[n++] = ptr;
			while ( *ptr && (*ptr != '"') ) ++ptr;
		} else {
			if ( *ptr )
				argv[n++] = ptr;
			while ( *ptr && ! isspace(*ptr) ) ++ptr;
		}
		if ( *ptr )
			*(ptr++) = '\0';
	}
	argv[n] = NULL;

	/* Get the class name from argv[0] */
	if ( (ptr=strrchr(argv[0], '\\')) == NULL )
		appname = argv[0];
	else
		appname = ptr+1;

	if ( (ptr=strrchr(appname, '.')) == NULL )
		n = strlen(appname);
	else
		n = (ptr-appname);

	if ( (ptr=(char *)malloc(n+1)) == NULL ) {
		/* Yuck, out of memory at startup */
		return(FALSE);
	}
	strncpy(ptr, appname, n);
	ptr[n] = '\0';
	appname = ptr;
	
	/* Load SDL dynamic link library */
	if ( SDL_Init(0) < 0 ) {
		fprintf(stderr, "WinMain() error: %s\n", SDL_GetError());
		return(FALSE);
	}
	atexit(SDL_Quit);

	/* Create and register our class, then run main code */
	if ( SDL_RegisterApp(appname, CS_BYTEALIGNCLIENT, hInst) < 0 ) {
		fprintf(stderr, "WinMain() error: %s\n", SDL_GetError());
		return(FALSE);
	}
	RunMain(argc, argv);

	exit(0);
}
