/*
    SDL - Simple DirectMedia Layer
    Copyright (C) 1997, 1998  Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Sam Lantinga
    5635-34 Springhouse Dr.
    Pleasanton, CA 94588 (USA)
    slouken@devolution.com
*/

#ifdef SAVE_RCSID
static char rcsid =
 "@(#) $Id: SDL_thread.c,v 1.1.1.1 1998/07/14 01:51:17 slouken Exp $";
#endif

/* Win32 thread management routines for SDL */

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>

#include "SDL_thread.h"


struct SDL_Thread {
	HANDLE id;
};

typedef struct {
	void (*func)(void *);
	void  *data;
} thread_args;

static DWORD WINAPI RunThread(LPVOID data)
{
	thread_args *args;
	void (*userfunc)(void *);
	void  *userdata;

	args = (thread_args *)data;
	userfunc = args->func;
	userdata = args->data;
	free(args);
	(*userfunc)(userdata);
	return(TRUE);
}

SDL_Thread *SDL_CreateThread(void (*fn)(void *), void *data)
{
	SDL_Thread *thread;
	HANDLE threadid;
	DWORD  threadnum;
	thread_args *args;

	/* Allocate memory for the thread info structure */
	thread = (SDL_Thread *)malloc(sizeof(*thread));
	if ( thread == NULL )
		return(NULL);

	/* Create the thread */
	args = (thread_args *)malloc(sizeof(*args));
	if ( args == NULL ) {
		free(thread);
		return(NULL);
	}
	args->func = fn;
	args->data = data;
	threadid = CreateThread(NULL, 0, RunThread, args, 0, &threadnum);
	if (threadid == NULL) {
		free(thread);
		return(NULL);
	}
	thread->id = threadid;
	return(thread);
}

void SDL_WaitThread(SDL_Thread *thread)
{
	if ( thread ) {
		WaitForSingleObject(thread->id, INFINITE);
		CloseHandle(thread->id);
		free(thread);
	}
}

/* WARNING: This function is really a last resort.
 * Threads should be signaled and then exit by themselves.
 * TerminateThread() doesn't perform stack and DLL cleanup.
 */
void SDL_KillThread(SDL_Thread *thread)
{
	if ( thread ) {
		TerminateThread(thread->id, FALSE);
		CloseHandle(thread->id);
		free(thread);
	}
}
