/*
   Byte-order routines -- placed in the public domain by Sam Lantinga
*/
#ifndef _bitesex_h
#define _bitesex_h

/* Grab the appropriate BYTE_ORDER definition */
#ifdef linux
#include <endian.h>
#else
#ifdef sparc
#ifdef __SVR4	/* Solaris */
#include <sys/byteorder.h>
#else		/* SunOS */
#define BYTE_ORDER	BIG_ENDIAN
#endif /* __SVR4 */
#else
#ifdef _SGI_SOURCE
#include <sys/endian.h>
#else
#ifdef _INCLUDE_HPUX_SOURCE
#include <machine/param.h>
#else
#ifdef __mips
#include <sex.h>
#else
#ifdef _AIX
#include <sys/machine.h>
#else
#ifdef __FreeBSD__
#include <machine/endian.h>
#else
#if defined(__svr4__)
#include <sys/byteorder.h>
#else
#if defined(__alpha__)
#include <alpha/endian.h>
#else
#ifdef _WIN32
#define LITTLEENDIAN
#else
#ifdef __BEOS__
#include <byteorder.h>
#define LITTLE_ENDIAN	1234
#define BIG_ENDIAN	4321
#if B_HOST_IS_LENDIAN
#define BYTE_ORDER	LITTLE_ENDIAN
#else
#define BYTE_ORDER	BIG_ENDIAN
#endif
#else
#error	Compiling for an unknown architecture!
#endif /* BeOS */
#endif /* Win32 */
#endif /* OSF1 */
#endif /* SVR4 */
#endif /* FreeBSD */
#endif /* AIX */
#endif /* DEC */
#endif /* HPUX */
#endif /* SGI */
#endif /* sparc */
#endif /* linux */

/* Okay, try double underscore */
#ifdef __BYTE_ORDER__
#define BYTE_ORDER	__BYTE_ORDER__
#define LITTLE_ENDIAN	__LITTLE_ENDIAN__
#define BIG_ENDIAN	__BIG_ENDIAN__
#else
/* Um, try double leading underscore */
#ifdef __BYTE_ORDER
#define BYTE_ORDER	__BYTE_ORDER
#define LITTLE_ENDIAN	__LITTLE_ENDIAN
#define BIG_ENDIAN	__BIG_ENDIAN
#endif /* __BYTE_ORDER */
#endif /* __BYTE_ORDER__ */

/* If we haven't gotten BYTE_ORDER by now, try to kludge it out */
#ifndef LITTLE_ENDIAN
#define LITTLE_ENDIAN	1234
#endif
#ifndef BIG_ENDIAN
#define BIG_ENDIAN	4321
#endif

#ifndef BYTE_ORDER
/* I don't remember which system this covers */
#ifdef _BIG_ENDIAN
#define BYTE_ORDER	BIG_ENDIAN
#endif
/* DEC -- Little Endian */
#ifdef LITTLEENDIAN
#define BYTE_ORDER LITTLE_ENDIAN
#endif
#endif /* No BYTE_ORDER yet */


#if BYTE_ORDER == LITTLE_ENDIAN

/* Macros to change the byte order of input values */
#define bytesex32(x)	(x = \
        ((x << 24) | ((x & 0xff00) << 8) | ((x >> 8) & 0xff00) | (x >> 24)))
#define bytesex16(x)	(x = (x << 8) | (x >> 8))

#else
#define bytesex32(x)	(x)
#define bytesex16(x)	(x)

#endif /* Endianness */

#ifndef BYTE_ORDER
#error I dont have any byte sex!
#endif

/* Swap bytes from big-endian to this machine's type.
   The input data is assumed to be always in big-endian format.
*/
static inline void
byteswap(Uint16 *array, int nshorts)
{
	for (; nshorts-- > 0; array++)
		*array = bytesex16(*array);
}

#endif /* _bitesex_h */
